{% spaceless %}
    <html>
        <head>
            <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
            <script type="{{ script_type }}">{{ payload|safe }}</script>
        </head>
    </html>
{% endspaceless %}